@@ -233,20 +233,23 @@ lookup_object_from_handle (GckSession *self, CK_OBJECT_HANDLE handle,
 	
 	if (handle == 0)
 		return CKR_OBJECT_HANDLE_INVALID;
+
+	/* Try looking up in the token manager */
+	manager = gck_module_get_manager (self->pv->module);
+	object = gck_manager_find_by_handle (manager, handle);
+	is_token = TRUE;
 	
-	if (handle & GCK_OBJECT_IS_PERMANENT) {
-		manager = gck_module_get_manager (self->pv->module);
-		is_token = TRUE;
-	} else {
+	/* Try looking up in the session manager */
+	if (object == NULL) {
 		manager = gck_session_get_manager (self);
+		object = gck_manager_find_by_handle (manager, handle);
 		is_token = FALSE;
 	}
 	
-	g_return_val_if_fail (manager, CKR_GENERAL_ERROR);
-	
-	object = gck_manager_find_by_handle (manager, handle);
 	if (object == NULL)
 		return CKR_OBJECT_HANDLE_INVALID;
+
+	g_return_val_if_fail (manager, CKR_GENERAL_ERROR);
 	
 	/* 
 	 * Check that we're not accessing private objects on a 
@@ -688,23 +691,6 @@ gck_session_C_GetSessionInfo(GckSession* self, CK_SESSION_INFO_PTR info)
 	return CKR_OK;
 }
 
-CK_RV
-gck_session_C_InitPIN (GckSession* self, CK_UTF8CHAR_PTR pin,
-                       CK_ULONG pin_len)
-{
-	/* We don't support this stuff. We don't support 'SO' logins. */
-	return CKR_USER_NOT_LOGGED_IN;	
-}
-
-CK_RV
-gck_session_C_SetPIN (GckSession* self, CK_UTF8CHAR_PTR old_pin,
-                      CK_ULONG old_pin_len, CK_UTF8CHAR_PTR new_pin, CK_ULONG new_pin_len)
-{
-	/* TODO: We may support this in the future. */
-	return CKR_FUNCTION_NOT_SUPPORTED;
-
-}
-
 CK_RV
 gck_session_C_GetOperationState (GckSession* self, CK_BYTE_PTR operation_state,
                                  CK_ULONG_PTR operation_state_len)
